self.__precacheManifest = (self.__precacheManifest || []).concat([
  {
    "revision": "81abc12e73996134b35f2d950746cc38",
    "url": "./index.html"
  },
  {
    "revision": "b95d6f4d051273edeaae",
    "url": "./static/css/main.1b9fbc2f.chunk.css"
  },
  {
    "revision": "a241b59f5e6fc058b1ca",
    "url": "./static/js/2.87df79f8.chunk.js"
  },
  {
    "revision": "a120daea67c98f2907b7f8aedcea87dd",
    "url": "./static/js/2.87df79f8.chunk.js.LICENSE.txt"
  },
  {
    "revision": "b95d6f4d051273edeaae",
    "url": "./static/js/main.f8ccf90e.chunk.js"
  },
  {
    "revision": "6515c66d2a8747a146d578e1c038a822",
    "url": "./static/js/main.f8ccf90e.chunk.js.LICENSE.txt"
  },
  {
    "revision": "7e9d84e346ce158d1e50",
    "url": "./static/js/runtime-main.44d30fc2.js"
  }
]);